
#include "MidiInput.hpp"
#include "KeyboardSynthesizer.hpp"

MidiInput::MidiInput(KeyboardSynthesizer* keyboardSynthesizer)
: midiInHandle_(nullptr), keyboardSynthesizer_(keyboardSynthesizer)
{
}

MidiInput::~MidiInput()
{
	if (midiInHandle_ != nullptr)
	{
		midiInStop(midiInHandle_);
		midiInClose(midiInHandle_);
	}
}

bool MidiInput::isOpened()
{
	return midiInHandle_ != nullptr;
}

LRESULT MidiInput::open()
{
	if (midiInHandle_ != nullptr)
		return 0; // ɃI[vĂ̂ŃIbP[

	UINT count = midiInGetNumDevs();
	if (count == 0)
		return ERROR_DEVICE_NOT_FOUND;

	if (midiInOpen(&midiInHandle_, 0, reinterpret_cast<DWORD_PTR>(callbackProcedure), reinterpret_cast<DWORD_PTR>(this), CALLBACK_FUNCTION) != MMSYSERR_NOERROR)
	{
		midiInHandle_ = nullptr;
		return ERROR_UNKNOWN;
	}

	if (midiInStart(midiInHandle_) != MMSYSERR_NOERROR)
	{
		midiInHandle_ = nullptr;
		return ERROR_UNKNOWN;
	}

	return 0;
}

void MidiInput::close()
{
	if (midiInHandle_ != nullptr)
	{
		midiInStop(midiInHandle_);
		midiInClose(midiInHandle_);
		midiInHandle_ = nullptr;
	}
}

void MidiInput::procedure(HMIDIIN hMidiIn, UINT wMsg, DWORD_PTR dwInstance, DWORD dwParam1, DWORD dwParam2)
{
	switch (wMsg)
	{
	case MIM_DATA:
	case MIM_LONGDATA:
		if ((dwParam1 & 0xf0) == 0x90)
			keyboardSynthesizer_->press((dwParam1) >> 8 & 0xff);
		break;
	}
}

void CALLBACK MidiInput::callbackProcedure(HMIDIIN hMidiIn, UINT wMsg, DWORD_PTR dwInstance, DWORD dwParam1, DWORD dwParam2)
{
	if (dwInstance == 0)
		return;
	else
		reinterpret_cast<MidiInput*>(dwInstance)->procedure(hMidiIn, wMsg, dwInstance, dwParam1, dwParam2);
}
